@@ -1,17 +1,17 @@
 # -*- coding: utf-8 -*-
 """
     Pygments basic API tests
-    ~~~~~~~~~~~~~~~~~~~~~~~
+    ~~~~~~~~~~~~~~~~~~~~~~~~
 
     :copyright: 2006 by Georg Brandl.
-    :license: GNU GPL, see LICENSE for more details.
+    :license: BSD, see LICENSE for more details.
 """
 
 import unittest
 import StringIO
 import random
 
-from pygments import lexers, formatters
+from pygments import lexers, formatters, format
 from pygments.token import _TokenType
 
 test_content = [chr(i) for i in xrange(33, 128)] * 5
@@ -20,6 +20,11 @@ test_content = ''.join(test_content) + '\n'
 
 class LexersTest(unittest.TestCase):
 
+    def test_import_all(self):
+        # instantiate every lexer, to see if the token type defs are correct
+        for x in lexers.LEXERS.keys():
+            c = getattr(lexers, x)()
+
     def test_lexer_classes(self):
         a = self.assert_
         ae = self.assertEquals
@@ -36,7 +41,9 @@ class LexersTest(unittest.TestCase):
             for token in tokens:
                 a(isinstance(token, tuple))
                 a(isinstance(token[0], _TokenType))
-                a(isinstance(token[1], str))
+                if isinstance(token[1], str):
+                    print repr(token[1])
+                a(isinstance(token[1], unicode))
                 txt += token[1]
             ae(txt, test_content, "%s lexer roundtrip failed: %r != %r" %
                     (lexer.name, test_content, txt))
@@ -67,14 +74,29 @@ class FormattersTest(unittest.TestCase):
         # test that every formatter class has the correct public API
         for formatter, info in formatters.FORMATTERS.iteritems():
             a(len(info) == 4)
-            a(info[0]) # name
-            a(info[1]) # aliases
-            a(info[3]) # doc
+            a(info[0], "missing formatter name") # name
+            a(info[1], "missing formatter aliases") # aliases
+            a(info[3], "missing formatter docstring") # doc
 
             inst = formatter(opt1="val1")
             inst.get_style_defs()
             inst.format(ts, out)
 
+    def test_unicode_handling(self):
+        # test that the formatter supports encoding and Unicode
+        tokens = list(lexers.PythonLexer(encoding='utf-8').get_tokens("def f(): 'ä'"))
+        for formatter, info in formatters.FORMATTERS.iteritems():
+            inst = formatter(encoding=None)
+            out = format(tokens, inst)
+            if formatter.unicodeoutput:
+                self.assert_(type(out) is unicode)
+
+            inst = formatter(encoding='utf-8')
+            out = format(tokens, inst)
+            self.assert_(type(out) is str)
+            # Cannot test for encoding, since formatters may have to escape
+            # non-ASCII characters.
+
     def test_get_formatters(self):
         a = self.assert_
         ae = self.assertEquals
